# GLADOS GUI test with absolute position button placement...

import Tkinter as ttk
from Tkinter import *
import tkMessageBox
import Tkinter, Tkconstants, tkFileDialog
import os
import glob
import subprocess
from subprocess import Popen, PIPE
import webbrowser

# Found on the internet. Lets us add hyperlinks to text.
class HyperlinkManager:
	def __init__(self, text):
		self.text = text
		self.text.tag_config("hyper", foreground="blue", underline=1, font=('Droid Sans Mono', 9))
		self.text.tag_bind("hyper", "<Enter>", self._enter)
		self.text.tag_bind("hyper", "<Leave>", self._leave)
		self.text.tag_bind("hyper", "<Button-1>", self._click)
		self.reset()

	def reset(self):
		self.links = {}

	def add(self, action):
		# Add an action to the manager. Returns tags to use in associated text widget
		tag = "hyper-%d" % len(self.links)
		self.links[tag] = action
		return "hyper", tag

	def _enter(self, event):
		self.text.config(cursor="hand2")

	def _leave(self, event):
		self.text.config(cursor="")

	def _click(self, event):
		for tag in self.text.tag_names(CURRENT):
			if tag[:6] == "hyper-":
				self.links[tag]()
				return



# Set current directory to script location
abspath = os.path.abspath(__file__)
dname = os.path.dirname(abspath)
os.chdir(dname)

# Create the main window widget, give it a name and set its size
MainWindow = Tk()
MainWindow.title('GLADoS')
MainWindow.geometry("500x600")
MainWindow.call('tk','scaling',1.2)

# HEADER TEXT
Title = Text(MainWindow, height=2, width=3)
Title.tag_configure('big', font=('Roboto', 20, 'bold'), justify='center')
Title.insert(END, 'GLADoS', 'big')

Title.insert(END, '\n')

# This is the default state on Linux. Hopefully this will work on Windows too !
Title.tag_configure('normal', font=('Droid Sans Mono', 9), justify='left')

maintext = """
GLADoS (Galaxy Line Analysis for Detection of Sources) 
is a source extractor designed for HI data cubes 
dominated  by point sources. It searches each spectrum
for peaks above a S/N and velocity width threshold, then
merges adjacent candidates to create a final catalogue 
and plots all the spectra. The user can then review them
to assign their status.
"""
Title.insert(END, maintext, 'normal')

maintext = """
When you press 'Go !', this window will disappear and
GLADos will search using your specified parameters.
Image files of the spectra will be created and you can
then review them. If you need to stop, use the 'Save &
Exit' button to safely exit. You can then resume at any 
time using the 'resume' mode.
"""
Title.insert(END, maintext, 'normal')

# Prevent the user from being able to edit the text
Title.config(state=DISABLED)

Title.place(x=50,y=10,width=400, heigh=260)


# CODE TO CREATE TOOLTIPS
class CreateToolTip(object):
	'''create a tooltip for a given widget'''
	def __init__(self, widget, text='widget info'):
		self.widget = widget
		self.text = text
		# "Bind" means to associate the widget with the specified events, and call the appropriate
		# functions in response.
		# Here the tooltip is activated when the mouse enters the button, and removed when the mouse
		# leaves or any mouse button is clicked
		self.widget.bind("<Enter>", self.enter)
		self.widget.bind("<Leave>", self.close)
		self.widget.bind("<ButtonPress>", self.close)		
	def enter(self, event=None):
		x = y = 0
		x, y, cx, cy = self.widget.bbox("insert")
		x += self.widget.winfo_rootx() + 25
		y += self.widget.winfo_rooty() + 20
		# Creates a toplevel window
		self.tw = ttk.Toplevel(self.widget)
		# Leaves only the label and removes the app window
		self.tw.wm_overrideredirect(True)
		self.tw.wm_geometry("+%d+%d" % (x, y))
		label = ttk.Label(self.tw, text=self.text, justify='left',background='light yellow', relief='solid', borderwidth=1,font=("Arial", "12", "normal"))
		label.pack(ipadx=1)
		# Remove after a ten second delay
		#self.tw.after(10000, self.tw.destroy)	
	def close(self, event=None):
		if self.tw:
			self.tw.destroy()
				


# DROP DOWN MENUS
# Create Tkinter variable for mode selection. String because our choices will be strings.
ModeVar = StringVar(MainWindow)

# Spectrum integration style variable
IntgVar = StringVar(MainWindow)

# Category of sources variable, for resuming
CatrVar  = StringVar(MainWindow)


# Create list of options for the mode selector
modes = ['Search','Review','Complete','Resume']

# Options for spectrum integration
intsl = ['Beam','Simple']

# Options for resume category
catrm = ['All','Accepted','Uncertain','Rejected']


# Set default options
ModeVar.set('Search')
IntgVar.set('Beam')
CatrVar.set('All')


# Attach the buttons to the main window. I think the asterisk is some sort of format specifier.
ModeMenu = OptionMenu(MainWindow, ModeVar, *modes)
CreateToolTip(ModeMenu, "Select what you want GLADoS to do.\n'Search' will run the search and create a single catalogue of candidates.\n'Review' will open spectra for candidates from an existing catalogue file and allow you to flag them.\n'Complete' does both the search and review.\n'Resume' mode allows you to resume reviewing from where you left off it you didn't finish reviewing the first search.")	

IntgMenu = OptionMenu(MainWindow, IntgVar, *intsl)
CreateToolTip(IntgMenu, "How to combine pixels.\n'Beam' weights surrounding pixels and accounts for the beam shape.\n'Simple' extracts spectra along individual pixels.")

CatrMenu = OptionMenu(MainWindow, CatrVar, *catrm)
CreateToolTip(CatrMenu, "Specify what category of sources to resume reviewing. Ignored unless mode is set to 'Review'.")
# Disable button by default
CatrMenu.config(state=DISABLED)

# Place buttons
ModeMenu.place(x=200,y=310,width=120)
IntgMenu.place(x=200,y=370,width=120)
CatrMenu.place(x=200,y=340,width=120)


# Checkbox to enable use of multiple polarisations
PolVar = IntVar()
UsePols = Checkbutton(MainWindow, text="  Dual polarisation", variable=PolVar)
CreateToolTip(UsePols, "Optionally use dual polarisations for the search.")
UsePols.place(x=48,y=345)

# Checkbox to enable use of advanced options, so new users won't be scared
AdvancedOps = IntVar()
DoAdvanced = Checkbutton(MainWindow, text="  Use advanced options", variable=AdvancedOps)
CreateToolTip(DoAdvanced, "Enable manually setting the parameters for baseline subtracting and catalogue matching")
DoAdvanced.place(x=48,y=435)


# Add labels
Label(MainWindow, text="Select FITS files").place(x=50, y=290)
Label(MainWindow, text="Search operations").place(x=200, y=290)
Label(MainWindow, text="Search settings").place(x=330, y=290)
Label(MainWindow, text="Configuration").place(x=50,y=460)
BLabel = Label(MainWindow, text="Baseline fitting")
BLabel.place(x=200,y=460)
BLabel.config(state=DISABLED)

SLabel = Label(MainWindow, text="Source matching")
SLabel.place(x=330,y=460)
SLabel.config(state=DISABLED)

PolyLabel = Label(MainWindow, text="Poly. order")
PolyLabel.place(x=200,y=485)
PolyLabel.config(state=DISABLED)

ClipLabel = Label(MainWindow, text="Clip value")
ClipLabel.place(x=200,y=515)
ClipLabel.config(state=DISABLED)

NitLabel = Label(MainWindow, text="No. iter.")
NitLabel.place(x=200,y=545)
NitLabel.config(state=DISABLED)

SpLabel = Label(MainWindow, text="Spatial pix.")
SpLabel.place(x=330,y=485)
SpLabel.config(state=DISABLED)

VCLabel= Label(MainWindow, text="Vel. channs.")
VCLabel.place(x=330,y=515)
VCLabel.config(state=DISABLED)

GLabel = Label(MainWindow, text="Group size")
GLabel.place(x=330,y=545)
GLabel.config(state=DISABLED)


# Operation of mode selector menu
def change_mode(*args):
	ModeVar.get()
	if ModeVar.get() == 'Review':
		CatrMenu.config(state=NORMAL)
	if ModeVar.get() == 'Resume':
		CatrVar.set('All')
		CatrMenu.config(state=DISABLED)
	if ModeVar.get() <> 'Review':
		CatrMenu.config(state=DISABLED)
    
# Operation of integration style selector menu
def integ_style(*args):
	IntgVar.get() 
	
# Operation of resume category selector menu
def catr_mode(*args):
	CatrVar.get()
	
# Grey out or activate certain options and labels depending on whether dual polarisation mode is enabled	
def enable_pols(*args):
	if PolVar.get() == 0:
		fitsfileAbox.config(state=DISABLED)
		fitsfileBbox.config(state=DISABLED)
		snpbox.config(state=DISABLED)
		vwpbox.config(state=DISABLED)
		vtolbox.config(state=DISABLED)
		VtolLab.config(state=DISABLED)
	if PolVar.get() == 1:
		fitsfileAbox.config(state=NORMAL)
		fitsfileBbox.config(state=NORMAL)
		snpbox.config(state=NORMAL)
		vwpbox.config(state=NORMAL)
		vtolbox.config(state=NORMAL)
		VtolLab.config(state=NORMAL)

# Enable or disable more advanced options
def enable_advanced(*args):
	if AdvancedOps.get() == 0:
		PolyLabel.config(state=DISABLED)
		polybox.config(state=DISABLED)
		clipbox.config(state=DISABLED)
		ClipLabel.config(state=DISABLED)
		NitLabel.config(state=DISABLED)
		niterbox.config(state=DISABLED)
		BLabel.config(state=DISABLED)
		SLabel.config(state=DISABLED)
		SpLabel.config(state=DISABLED)
		spaxbox.config(state=DISABLED)
		VCLabel.config(state=DISABLED)
		vchansbox.config(state=DISABLED)
		GLabel.config(state=DISABLED)
		gszebox.config(state=DISABLED)
	if AdvancedOps.get() == 1:
		PolyLabel.config(state=NORMAL)
		polybox.config(state=NORMAL)
		clipbox.config(state=NORMAL)	
		ClipLabel.config(state=NORMAL)
		NitLabel.config(state=NORMAL)
		niterbox.config(state=NORMAL)
		BLabel.config(state=NORMAL)
		SLabel.config(state=NORMAL)
		SpLabel.config(state=NORMAL)
		spaxbox.config(state=NORMAL)
		VCLabel.config(state=NORMAL)
		vchansbox.config(state=NORMAL)
		GLabel.config(state=NORMAL)
		gszebox.config(state=NORMAL)
	 	
# Link functions to change dropdown. 'w' means to call the function when the variable is changed, i.e.
# when the button is pressed
ModeVar.trace('w', change_mode)
IntgVar.trace('w', integ_style)
CatrVar.trace('w', catr_mode)
PolVar.trace('w',enable_pols)
AdvancedOps.trace('w',enable_advanced)


# CREATE TEXT ENTRY BOXES
# Input FITS file name through text
# Get default filename. If no FITS file found, give it something instructive to the user
try:
	defaultfits = glob.glob('*.fits')[0]
except:
	defaultfits = 'Main FITS file'
fitsname = StringVar()
fitsfilebox = Entry(MainWindow, textvariable=fitsname)
fitsfilebox.insert(0,defaultfits)
fitsfilebox.place(x=75, y=310, width=110,height=28)
CreateToolTip(fitsfilebox, "Manually specify FITS file name with or without the full path.\n If only the filename is specified, GLADoS will assume it's in the current directory.")
fullfitsname = dname+'/'+defaultfits


# FITS file for polarisation A
try:
	defaultAfits = glob.glob('*_A.fits')[0]
except:
	defaultAfits ='Polarisation A'
fitsAname = StringVar()
fitsfileAbox = Entry(MainWindow, textvariable=fitsAname)
fitsfileAbox.insert(0,defaultAfits)
fitsfileAbox.place(x=75, y=370, width=110,height=28)
CreateToolTip(fitsfileAbox, "Manually specify FITS file name for polarisation A with or without the full path.\n If only the filename is specified, GLADoS will assume it's in the current directory.")
fullfitsAname = dname+'/'+defaultAfits
# Disable by default since we will usually only use a single cube. This needs to be done here,
# after the search for standard-named files, in order for the name to be shown when disabled.
fitsfileAbox.config(state=DISABLED)

# FITS file for polarisation B
try:
	defaultBfits = glob.glob('*_B.fits')[0]
except:
	defaultBfits ='Polarisation B'	
fitsBname = StringVar()
fitsfileBbox = Entry(MainWindow, textvariable=fitsBname)
fitsfileBbox.insert(0,defaultBfits)
fitsfileBbox.place(x=75, y=400, width=110,height=28)
CreateToolTip(fitsfileBbox, "Manually specify FITS file name for polarisation B with or without the full path.\n If only the filename is specified, GLADoS will assume it's in the current directory.")
fullfitsBname = dname+'/'+defaultBfits
fitsfileBbox.config(state=DISABLED)

# S/N threshold - main
snlevel = StringVar()
textbox = Entry(MainWindow, textvariable=snlevel)
textbox.insert(0,'4.0')
textbox.place(x=360,y=340, width=40, height=28)
Label(MainWindow, text="S/N").place(x=330, y=345)
CreateToolTip(textbox, "S/N  threshold for candidate detection in the main cube")

# S/N threshold - individual polarisations
snplevel = StringVar()
snpbox = Entry(MainWindow, textvariable=snplevel)
snpbox.insert(0,'3.5')
snpbox.place(x=401,y=340, width=40, height=28)
CreateToolTip(snpbox, "Optional S/N  threshold for candidate detection in individual polarisations")
snpbox.config(state=DISABLED)

# VW threshold - main cube
vwthresh = StringVar()
textbox = Entry(MainWindow, textvariable=vwthresh)
textbox.insert(0,'20.0')
textbox.place(x=360,y=370, width=40, height=28)
Label(MainWindow, text="VW").place(x=330, y=375)
CreateToolTip(textbox, "Velocity width threshold (km/s) for candidate detection in the main cube")

# VW threshold - individual polarisations
vwpthresh = StringVar()
vwpbox = Entry(MainWindow, textvariable=vwpthresh)
vwpbox.insert(0,'20.0')
vwpbox.place(x=401,y=370, width=40, height=28)
Label(MainWindow, text="VW").place(x=330, y=375)
CreateToolTip(vwpbox, "Optional velocity width threshold (km/s) for candidate detection in individual polarisations")
vwpbox.config(state=DISABLED)

# Velocity tolerance - only if use individual polarisations
vtol = StringVar()
vtolbox = Entry(MainWindow, textvariable=vtol)
vtolbox.insert(0,'1200.0')
vtolbox.place(x=385,y=400, width=55, height=28)
vtolbox.config(state=DISABLED)

VtolLab = Label(MainWindow, text="Vel. tol.")
VtolLab.place(x=330, y=405)
VtolLab.config(state=DISABLED)
CreateToolTip(vtolbox, "Velocity range within with a peak in the individual polarisations is considered an acceptable match for a detection in the main cube")


#Vmin
vmin = StringVar()
textbox = Entry(MainWindow, textvariable=vmin)
textbox.insert(0,'Vmin')
textbox.place(x=360,y=310, width=40, height=28)
CreateToolTip(textbox, "Minimum velocity (km/s) of search range.\nLeave as Vmin to use the lowest value in the cube.")
Label(MainWindow, text="Vel.").place(x=330, y=315)

#Vmax
vmax = StringVar()
textbox = Entry(MainWindow, textvariable=vmax)
textbox.insert(0,'Vmax')
textbox.place(x=401,y=310, width=40, height=28)
CreateToolTip(textbox, "Maximum velocity (km/s) of search range.\nLeave as Vmax to use the highest value in the cube.")


# Polynomial order
poly = StringVar()
polybox = Entry(MainWindow, textvariable=poly)
polybox.insert(0,'2')
polybox.place(x=270,y=480,width=50,height=28)
CreateToolTip(polybox, "Order of the polynomial to fit along the spectral baseline")
polybox.config(state=DISABLED)

# Clip value
clip = StringVar()
clipbox = Entry(MainWindow, textvariable=clip)
clipbox.insert(0,'2.0')
clipbox.place(x=270,y=510,width=50,height=28)
CreateToolTip(clipbox, "S/N value to clip in the iterative rms estimation")
clipbox.config(state=DISABLED)

# Number of iterations
niter = StringVar()
niterbox = Entry(MainWindow, textvariable=niter)
niterbox.insert(0,'5')
niterbox.place(x=270,y=540,width=50,height=28)
CreateToolTip(niterbox, "Number of iterations for sigma clipping in rms estimation")
niterbox.config(state=DISABLED)


# Spatial pixel distance for group matching
spax = StringVar()
spaxbox = Entry(MainWindow, textvariable=spax)
spaxbox.insert(0,'3')
spaxbox.place(x=410,y=480,width=30,height=28)
CreateToolTip(spaxbox, "Spatial pixel range to consider candidates as part of the same group")
spaxbox.config(state=DISABLED)

# Velocity channel distance for group matching
vchans = StringVar()
vchansbox = Entry(MainWindow, textvariable=vchans)
vchansbox.insert(0,'27')
vchansbox.place(x=410,y=510,width=30,height=28)
CreateToolTip(vchansbox, "Velocity channel range to consider candidates as part of the same group")
vchansbox.config(state=DISABLED)

# Minimium size of groups to examine
gsze = StringVar()
gszebox = Entry(MainWindow, textvariable=gsze)
gszebox.insert(0,'3')
gszebox.place(x=410,y=540,width=30,height=28)
CreateToolTip(gszebox, "Minimum number of candidates in a group")
gszebox.config(state=DISABLED)


# Java file
# First attempt to find file location automatically by calling the Linux "which" command
# Subprocess module has weird syntax which doesn't allow space-separated components
try:
	output=subprocess.Popen(["which", "stilts"],stdout=PIPE)
	response=output.communicate()[0].rstrip("r\n")	# The rstrip thing removes line breaks
	jsplit = response.split('/')
	jdir = ''
	for i in range(1,len(jsplit)-1):
		jdir = jdir + '/' + jsplit[i]
	jfile = jsplit[len(jsplit)-1]
	
except:
	jdir = ''
	response = ''

# If the above doesn't work, it will return blank values for jdir and jdisp. Replace these
# with something more useful for testing and display.
if jdir == '':
	jdir = 'XXX'
	jfile = 'File not found !'

jdisp = StringVar()
jdispbox = Entry(MainWindow, textvariable=jdisp)
jdispbox.insert(0, jfile)
jdispbox.place(x=75,y=480,width=110,height=28)
CreateToolTip(jdispbox, "Name of the STILTS file - will show an error if not found")
jdispbox.config(state=DISABLED)
jfullfilename = response	



# Get the values if the user updates any of the textboxes
# Not sure all of these are necessary. Values are set by the value in the box anyway. Some need other
# alterations so they can stay, but the one-liners can probably all be disabled.

# Function to get the FITS filename if the user edits the text box
# 'Setmethod' is a check for what method was used when the function was called. Set to be 'GUI' when using
# the GUI method.
setmethod = None
def update_fitsname(*args):
	global fullfitsname, setmethod
	filename = fitsname.get()
	
	# If the change has ocurred using the buttons, no need to alter the full file name. If the user typed
	# it manually, then we'll need to add in the full path if they didn't include it.
	if setmethod <> 'GUI':	
		if '/' in filename:
			fullfitsname = filename
		# If the user only types the file name, assume it's supposed to be in this directory
		if '/' not in filename:
			fullfitsname = dname+'/'+filename
	# Reset this to prevent it being stuck in GUI mode even when the user tries to type directly
	setmethod = None
	
# Call the function whenever the variable is altered	
fitsname.trace('w',update_fitsname)


# Similar procedure for the individual polarisations
setmethod = None
def update_fitsAname(*args):
	global fullfitsAname, setmethod
	filename = fitsAname.get()
	
	# If the change has ocurred using the buttons, no need to alter the full file name. If the user typed
	# it manually, then we'll need to add in the full path if they didn't include it.
	if setmethod <> 'GUI':	
		if '/' in filename:
			fullfitsAname = filename
		# If the user only types the file name, assume it's supposed to be in this directory
		if '/' not in filename:
			fullfitsAname = dname+'/'+filename
	setmethod = None
			
fitsAname.trace('w',update_fitsAname)

setmethod = None
def update_fitsBname(*args):
	global fullfitsBname, setmethod
	filename = fitsBname.get()
	
	# If the change has ocurred using the buttons, no need to alter the full file name. If the user typed
	# it manually, then we'll need to add in the full path if they didn't include it.
	if setmethod <> 'GUI':	
		if '/' in filename:
			fullfitsBname = filename
		# If the user only types the file name, assume it's supposed to be in this directory
		if '/' not in filename:
			fullfitsBname = dname+'/'+filename
	setmethod = None
			
fitsBname.trace('w',update_fitsBname)



# Same for S/N threshold
def update_snlevel(*args):
	snlev = snlevel.get()
	
snlevel.trace('w',update_snlevel)


# And now for VW threshold
def update_vwthresh(*args):
	vwval = vwthresh.get()
	
vwthresh.trace('w',update_vwthresh)


# Get Vmin from textbox
def update_vmin(*args):
	vminval = vmin.get()
	
vmin.trace('w',update_vmin)


# Get Vmax from textbox
def update_vmax(*args):
	vmaxval = vmax.get()
	
vmax.trace('w',update_vmax)


# Get clip from textbox
def update_clip(*args):
	clipval = clip.get()
	
clip.trace('w',update_clip)


# Get niter from textbox
def update_niter(*args):
	nitval = niter.get()
	
niter.trace('w',update_niter)


# Get spax from textbox
def update_spax(*args):
	spval = spax.get()
	
spax.trace('w',update_spax)


# Get vchans from textbox
def update_vchans(*args):
	nvc = vchans.get()
	
vchans.trace('w',update_vchans)


# Function to get the Java filename if the user edits the text box
setmethod = None
def update_jname(*args):
	global jfullfilename, setmethod
	jfilename = jdisp.get()
		
	if setmethod <> 'GUI':
		if '/' in jfilename:
			jfullfilename = jfilename
		# If the user only types the file name, assume it's supposed to be in this directory
		if '/' not in jfilename:
			jfullfilename = dname+'/'+jfilename
	setmethod = None
	
# Call the function whenever the variable is altered	
jdisp.trace('w',update_jname)


# Get polynomial order from textbox
def update_poly(*args):
	polyval = poly.get()

poly.trace('w',update_poly)


# Get group size from textbox
def update_gsze(*args):
	ngsze = gsze.get()

gsze.trace('w',update_gsze)



# Validate values
def validate():
	# Assume values are good. Reset them if a single bad value is found.
	validvalues = 1
	# Text message to be displayed in the event of an error
	errortext = ""
	
	# Check S/N 
	snlev = snlevel.get()
	try:
		test = float(snlev)
		if test < 1.0:
			raise
	except:
		print 'ERROR ! Invalid S/N level'
		validvalues = 0
		errortext = errortext +'\nInvalid S/N level, must be real number greater than 1.0'
	
	# Check S/N in individual polarisations
	snpval = snplevel.get()
	try:
		test = float(snpval)
		if test < 1.0:
			raise
	except:
		print 'ERROR ! Invalid S/N pol. level'
		validvalues = 0
		errortext = errortext +'\nInvalid S/N pol. level, must be real number greater than 1.0'
	
	# Check VW
	vwval = vwthresh.get()
	try:
		test = float(vwval)
		if test < 0.0:
			raise
	except:
		print 'ERROR ! Invalid VW level'
		validvalues = 0
		errortext = errortext +'\nInvalid VW level, must be positive real number'
		
	# Check VW for individual polarisations	
	vwpval = vwpthresh.get()
	try:
		test = float(vwpval)
		if test < 0.0:
			raise
	except:
		print 'ERROR ! Invalid VW pol. level'
		validvalues = 0
		errortext = errortext +'\nInvalid VW pol. level, must be positive real number'		
	
	
	# Check Vmin. Add a special check value because we'll also be comparing this with vmax.
	# Also has an exception for the "Vmin" string, which sets it to the edge of the cube.
	vminval = vmin.get()
	validvmin = 1
	# Special exception to allow the user to enter a string for Vmin, so that they can do
	# the full velocity range without knowing what it actually is
	if 'Vmin' not in vminval:
		try:
			test = float(vminval)
		except:
			print 'ERROR ! Invalid Vmin level, must be real number'
			validvalues = 0
			validvmin = 0
			errortext = errortext +'\nInvalid Vmin level'
	
		
	# Check Vmax. Add a special check value because we'll also be comparing this with vmin.
	# Also has an exception for the "Vmax" string, which sets it to the edge of the cube.
	vmaxval = vmax.get()
	validvmax = 1
	# Special exception to allow the user to enter a string for Vmax, so that they can do
	# the full velocity range without knowing what it actually is
	if 'Vmax' not in vmaxval:
		try:
			test = float(vmaxval)
		except:
			print 'ERROR ! Invalid Vmax level'
			validvalues = 0
			validvmax = 0
			errortext = errortext +'\nInvalid Vmax level'
		
	# Check that vmin < vmax if values are valid
	if validvmax == 1 and validvmin == 1 and'Vmin' not in vminval and 'Vmax' not in vmaxval:
		if float(vmaxval) <= float(vminval):
			print 'ERROR ! Vmax must be greater than Vmin'
			validvalues = 0
			errortext = errortext +'\nVmax must exceed Vmin'
	
	
	# Check velocity tolerance
	vtolval = vtol.get()	
	try:
		test = float(vtolval)
	except:
		print 'ERROR ! Invalid velocity tolerance value'
		validvalues = 0
		errortext = errortext +'\nInvalid velocity tolerance value, must be real number'	
	
	# Check poly order
	polyval = poly.get()
	try:
		test = int(polyval)
	except:
		print 'ERROR ! Invalid polynomial order level'
		validvalues = 0
		errortext = errortext +'\nInvalid polynomial order level, must be integer'
		
	# Check clip value
	clipval = clip.get()
	try:
		test = float(clipval)
	except:
		print 'ERROR ! Invalid clip level'
		validvalues = 0
		errortext = errortext +'\nInvalid clip level, must be real number'
	
	# Check niter value	
	nitval = niter.get()
	try:
		test = int(nitval)
		if test < 0:
			raise
	except:
		print 'ERROR ! Invalid no. iterations'
		validvalues = 0
		errortext = errortext +'\nInvalid no. iterations, must be positive integer'
		
	# Check spatial pixels value
	spval = spax.get()
	try:
		test = int(spval)
		if test < 0:
			raise
	except:
		print 'ERROR ! Invalid no. spatial pixels'
		validvalues = 0
		errortext = errortext +'\nInvalid no. spatial pixels, must be positive integer'
		
	# Check channels value
	nvc = vchans.get()
	try:
		test = int(nvc)
		if test < 0:
			raise
	except:
		print 'ERROR ! Invalid no. velocity channels'
		validvalues = 0
		errortext = errortext +'\nInvalid no. velocity channels, must be positive integer'	
	
	# Check group size value
	ngsze = gsze.get()
	try:
		test = int(ngsze)
		if test < 0:
			raise
	except:
		print 'ERROR ! Invalid group size'
		validvalues = 0
		errortext = errortext +'\nInvalid group size, must be positive integer'	
	
	if validvalues == 0:
		tkMessageBox.showerror("OH CRAP !", errortext)

	# Returns 1 if good, 0 if bad
	return validvalues


# Save settings to a file, either when running the main code (always saved to a specific file) or to a
# user-specified file from the GUI.
# Contains an internal call to the validate subroutine to definitely make sure the user doesn't do
# anything daft.
# Also makes sure that the fomrat of the variables is correct, i.e. 5.0 is written as 5 if it needs to
# be an integer; 5 is written as 5.0 is it needs to be real.
def savesettingsfile(settingsfilename):
	goodvalues = validate()
	if goodvalues == 1:
		setfile = open(settingsfilename,'w')
		setfile.write('MainFITSFile='+str(fullfitsname)+'\n')#fitsname.get())+'\n')
		setfile.write('DualPol='+str(PolVar.get())+'\n')
		setfile.write('PolAFITSFile='+str(fullfitsAname)+'\n')#fitsname.get())+'\n')
		setfile.write('PolBFITSFile='+str(fullfitsBname)+'\n')#fitsname.get())+'\n')
		setfile.write('Mode='+str(ModeVar.get())+'\n')
		setfile.write('Category='+str(CatrVar.get())+'\n')
		setfile.write('SN='+str(float(snlevel.get()))+'\n')
		setfile.write('S/N_pol='+str(float(snplevel.get()))+'\n')
		setfile.write('VW='+str(float(vwthresh.get()))+'\n')
		setfile.write('VW_pol='+str(float(vwpthresh.get()))+'\n')
		setfile.write('VelTol='+str(float(vtol.get()))+'\n')
		setfile.write('Integration='+str(IntgVar.get())+'\n')
		if 'Vmin' in vmin.get():
			setfile.write('Vmin='+str(vmin.get())+'\n')
		if 'Vmin' not in vmin.get():
			setfile.write('Vmin='+str(float(vmin.get()))+'\n')
		if 'Vmax' in vmax.get():
			setfile.write('Vmax='+str((vmax.get())+'\n'))
		if 'Vmax' not in vmax.get():
			setfile.write('Vmax='+str(float(vmax.get()))+'\n')
		setfile.write('AdvancedOps='+str(AdvancedOps.get())+'\n')	
		setfile.write('Jfile='+str(jfullfilename)+'\n')
		setfile.write('Poly='+str(int(poly.get()))+'\n')
		setfile.write('Clip='+str(float(clip.get()))+'\n')
		setfile.write('Niter='+str(int(niter.get()))+'\n')
		setfile.write('Spx='+str(int(spax.get()))+'\n')
		setfile.write('Nchns='+str(int(vchans.get()))+'\n')
		setfile.write('Gsize='+str(int(gsze.get()))+'\n')
		setfile.close()


# ONE CLICK BUTTONS
# I can't figure out how to get these to work without this horrible class structure but at least this 
# works.		
class ClickButtons():
	def __init__(self,master):
		frame = Frame(master)
		frame.pack()
				
		# Get FITS file button		
		self.getfitsfile = Button(MainWindow, text='O', command=self.getfitsfile)
		self.getfitsfile.place(x=50,y=310, width=25)	
		CreateToolTip(self.getfitsfile, "Select main FITS file")
		
		# Execute search !
		self.go = Button(MainWindow,text='GO !', command=self.go, font = ('Arial','12','bold', 'italic'))
		self.go.place(x=201,y=400,width=118, height=28)
		CreateToolTip(self.go, "Validate values and run the search !\nRegardless of whether you save the settings elsewhere or not,\nGLADos also creates a special file 'ActualSettingsUsed.txt',\nwhich saves the settings used in the search.")
		
		# Get JAVA file button
		self.getjavafile = Button(MainWindow, text='O', command=self.getjavafile)
		self.getjavafile.place(x=50,y=480, width=25)
		self.getjavafile.config(state=DISABLED)
		CreateToolTip(self.getjavafile, "Locate the Starlink 'STILTS' binary")
		
		# Load settings file button
		self.loadsettings = Button(MainWindow, text='Load settings', command=self.loadsettings)
		self.loadsettings.place(x=50,y=510, width=135)
		CreateToolTip(self.loadsettings, "Load search settings from a file")
		
		# Save settings file button
		self.savesettings = Button(MainWindow, text='Save settings', command=self.savesettings)
		self.savesettings.place(x=50,y=540, width=135)
		CreateToolTip(self.savesettings, "Save search settings to a file")
		
		# Get polarisation A FITS file button		
		self.getAfile = Button(MainWindow, text='O', command=self.getAfile)
		self.getAfile.place(x=50,y=370, width=25)
		self.getAfile.config(state=DISABLED)	# Disabled by default
		CreateToolTip(self.getAfile, "Select FITS file for polarisation A")
		
		# Get polarisation B FITS file button		
		self.getBfile = Button(MainWindow, text='O', command=self.getBfile)
		self.getBfile.place(x=50,y=400, width=25)
		self.getBfile.config(state=DISABLED)	# Disabled by default
		CreateToolTip(self.getBfile, "Select FITS file for polarisation B")
		
		# Show some more detailed instructions
		self.instructions = Button(MainWindow, text='?', command=self.help_window, font = ('Arial','12','bold'))
		self.instructions.place(x=425,y=245, width=20, height=20)	
		CreateToolTip(self.instructions, "More detailed instructions")
		
		# Although the effects of the polarisation button are already set elsewhere, need a separate
		# call to thise inside this module since buttons are effectively local variables	
		def enable_pols_buttons(*args):
			if PolVar.get() == 0:
				self.getAfile.config(state=DISABLED)
				self.getBfile.config(state=DISABLED)
			if PolVar.get() == 1:
				self.getAfile.config(state=NORMAL)
				self.getBfile.config(state=NORMAL)		
		
		PolVar.trace('w',enable_pols_buttons)
		
		# Similarly we need to check the advanced options button here as well
		def enable_advanced(*args):
			if AdvancedOps.get() == 0:
				self.getjavafile.config(state=DISABLED)
				jdispbox.config(state=DISABLED)
			if AdvancedOps.get() == 1:
				self.getjavafile.config(state=NORMAL)
				jdispbox.config(state=NORMAL)
				
		AdvancedOps.trace('w',enable_advanced)
		
	
	# FITS file button operation	
	def getfitsfile(self):
		global fullfitsname, setmethod
		fullfitsname = tkFileDialog.askopenfilename(initialdir=dname,title = "Select file",filetypes = (("FITS files","*.fits"),("All files","*.*")))		
		setmethod = 'GUI'
		nameparts = fullfitsname.split('/')
		shortname = nameparts[len(nameparts)-1]
		
		# Update the variable that's actually dispalyed
		fitsname.set(shortname)		
		
		# Remember that shortname is for display purposes. The full name with path is fullfitsname.
	
	
	# JAVA file button operations
	def getjavafile(self):
		global jfullfilename, setmethod
		setmethod = 'GUI'
		# For some reason if we only want to allow all files we have to specify it twice
		jfullfilename = tkFileDialog.askopenfilename(initialdir=dname,title = "Select file",filetypes = (("All files","*.*"),("All files","*.*")))
		jnameparts = jfullfilename.split('/')
		jshortname = jnameparts[len(jnameparts)-1]
				
		jdisp.set(jshortname)
		
		

	def loadsettings(self):
		fullfilename = tkFileDialog.askopenfilename(initialdir=dname,title = "Select file",filetypes = (("Text files","*.txt"),("All files","*.*")))
		if fullfilename is None:
			return
		settingsfile = open(fullfilename,'r')
		# File will always have the same format, so we can just proceed line by line
		fitsfilename = settingsfile.readline().splitlines()[0].split('=')[1]
		fitsname.set(fitsfilename)
		
		dualpol = settingsfile.readline().splitlines()[0].split('=')[1]
		PolVar.set(int(dualpol))

		fitsAfilename = settingsfile.readline().splitlines()[0].split('=')[1]
		fitsAname.set(fitsAfilename)
		
		fitsBfilename = settingsfile.readline().splitlines()[0].split('=')[1]
		fitsBname.set(fitsBfilename)

		mode = settingsfile.readline().splitlines()[0].split('=')[1]
		ModeVar.set(mode)

		catg = settingsfile.readline().splitlines()[0].split('=')[1]
		CatrVar.set(catg)
		
		sn = settingsfile.readline().splitlines()[0].split('=')[1]
		snlevel.set(sn)
		
		snp = settingsfile.readline().splitlines()[0].split('=')[1]
		snplevel.set(snp)
		
		vw = settingsfile.readline().splitlines()[0].split('=')[1]
		vwthresh.set(vw)
		
		vwp = settingsfile.readline().splitlines()[0].split('=')[1]
		vwpthresh.set(vwp)
		
		vtolval = settingsfile.readline().splitlines()[0].split('=')[1]
		vtol.set(vtolval)		
		
		Intg = settingsfile.readline().splitlines()[0].split('=')[1]
		IntgVar.set(Intg)
		
		lowv = settingsfile.readline().splitlines()[0].split('=')[1]
		vmin.set(lowv)
		
		highv = settingsfile.readline().splitlines()[0].split('=')[1]
		vmax.set(highv)

		adops = settingsfile.readline().splitlines()[0].split('=')[1]
		AdvancedOps.set(int(adops))
		
		jfile = settingsfile.readline().splitlines()[0].split('=')[1]
		jdisp.set(jfile)
		
		porder = settingsfile.readline().splitlines()[0].split('=')[1]
		poly.set(porder)
		
		cval = settingsfile.readline().splitlines()[0].split('=')[1]
		clip.set(cval)
		
		nt = settingsfile.readline().splitlines()[0].split('=')[1]
		niter.set(nt)
		
		spx = settingsfile.readline().splitlines()[0].split('=')[1]
		spax.set(spx)
		
		nc = settingsfile.readline().splitlines()[0].split('=')[1]
		vchans.set(nc)
		
		gs = settingsfile.readline().splitlines()[0].split('=')[1]
		gsze.set(gs)
		 
	
	# This button calls another subroutine which does the actual file-saving. This is because when we
	# run the main code we want to be certain the settings have been saved to a file of known name, so
	# that GLADOS itself can reliably call it.	 
	def savesettings(self):
		settingsfile = tkFileDialog.asksaveasfile(initialdir=dname, mode='w', defaultextension=".txt")
		if settingsfile is None:
			return
		# GUI returns a file object, get the name to send it to the file-writing subroutine
		settingsfilename = settingsfile.name
		# Close the file to avoid any possible conflicts. Doesn't matter than the saving subroutine will
		# just reopen it.
		settingsfile.close()
		savesettingsfile(settingsfilename)

	# Get the FITS file for polarisation A
	def getAfile(self):
		global fullfitsAname, setmethod
		if PolVar.get() == 1:
			fullfitsAname = tkFileDialog.askopenfilename(initialdir=dname,title = "Select file",filetypes = (("FITS files","*.fits"),("All files","*.*")))		
			setmethod = 'GUI'
			nameparts = fullfitsAname.split('/')
			shortname = nameparts[len(nameparts)-1]
		
			# Update the variable that's actually dispalyed
			fitsAname.set(shortname)			

	# Get the FITS file for polarisation B
	def getBfile(self):
		global fullfitsBname, setmethod
		if PolVar.get() == 1:
			fullfitsBname = tkFileDialog.askopenfilename(initialdir=dname,title = "Select file",filetypes = (("FITS files","*.fits"),("All files","*.*")))		
			setmethod = 'GUI'
			nameparts = fullfitsBname.split('/')
			shortname = nameparts[len(nameparts)-1]
		
			# Update the variable that's actually dispalyed
			fitsBname.set(shortname)


	def help_window(self):	
		helpwindow = ttk.Toplevel(MainWindow)
		helpwindow.title('GLADoS Instructions')
		helpwindow.geometry("500x500")
		
		scrollbar = Scrollbar(helpwindow)
		scrollbar.pack(side=RIGHT, fill=Y)
		
		screen_width = MainWindow.winfo_screenwidth()
		screen_height = MainWindow.winfo_screenheight()

    	# Calculate position x and y coordinates to ensure it's always in the centre of the screen
		width = 500
		height = 500
		x = (screen_width/2) - (width/2)
		y = (screen_height/2) - (height/2)
		helpwindow.geometry('%dx%d+%d+%d' % (width, height, x, y))
		
		def addheadlink():		
			url='http://www.rhysy.net/Resources/Programs/AddHeader.py'
			webbrowser.open(url,new=1)
		
		def starlinklink():
			url='http://www.star.bris.ac.uk/~mbt/stilts/'
			webbrowser.open(url,new=1)
		
		# HEADER TEXT
		NewTitle = Text(helpwindow, height=2, width=3, yscrollcommand=scrollbar.set)
		NewTitle.tag_configure('big', font=('Roboto', 16, 'bold'), justify='center')
		NewTitle.insert(END, 'A brief guide to using GLADoS', 'big')
		NewTitle.insert(END, '\n')
		NewTitle.insert(END, '\n')
		
		NewTitle.tag_configure('subsection', font=('Arial', 11, 'bold','underline'))
		NewTitle.insert(END, 'Before you begin', 'subsection')

		NewTitle.tag_configure('normal', font=('Droid Sans Mono', 9), justify='left')

		maintext = """
GLADoS can operate either on individual cubes or, if available, 
using dual polarisation data. 
If searching a single cube, most of the parameters should be self
explantory. Either way, there are three main points to remember :
 
1) GLADoS does not work well in regions strongly affected by RFI. 
The easiest way to deal with these is to mask them before searching
using an external program such as miriad.

2) Ideally, search the entire velocity range of the cube at once.
This is much easier if corrupted regions have been masked first, 
but sometimes it can be necessary to search more limited velocity
segments. If so, remember that each catalogue generated by GLADoS 
will not account for the naming scheme of the others - you will
need to rename both the catalogue files and the source numbers to
have a consistent scheme.

3) The main FITS cube needs to have a header describing the beam
size (e.g. BMAJ, BMAJOR, BEAMMAJOR, with the value given in 
degrees). This is needed for extracting the spectrum if surroudning
pixels are weighted according to the beam size. It's also used for 
the image maps and the SDSS and NED query buttons. GLADoS will warn 
you if it can't find any of the the standard keywords. If needed, 
you can add or update header key words with """
		NewTitle.insert(END, maintext, 'normal')
		
		# Attached the hyperlinke manager to the current text block and add the first link
		hyperlink = HyperlinkManager(NewTitle)
		NewTitle.insert(END, "this code.", hyperlink.add(addheadlink))
				
		NewTitle.insert(END, '\n')
		
		maintext = """	
A more minor point is that when reviewing an existing catalogue,
although GLADoS does not need any parameters from the original
data cube, it still expects a FITS file to be present. So you need
to have a FITS file even if you don't want to actually search it."""
		NewTitle.insert(END, maintext, 'normal')
		
		NewTitle.insert(END, '\n')
		NewTitle.insert(END, '\n')
		NewTitle.insert(END, '\n')	
	
		NewTitle.insert(END, 'Working with dual polarisation data', 'subsection')
		maintext = """
If you have dual polarisation data available, using this can
significantly increase reliability by eliminating spurious spikes 
in the noise that appear in only one polarisation. ALL cubes MUST 
have identical pixel and world coordinates in all axes. GLADoS
proceeds by first checking the main cube for a detection and then
inspecting the individual polarisations at the same coordinates.

Since individual polarisations have higher noise levels than the
averaged data, it can be useful to smooth and/or use a lower S/N 
and  velocity width threshold for the individual cubes. Smoothing
must be done in an external program.
The 'velocity tolerance' parameter is designed for sources with
broad velocity widths and low peak flux, where it is possible that
different peaks will be detected in each polarisation. If a source
is found in the main cube, GLADoS will accept it as detected in the
individual polarisations if a corresponding peak is found within 
the specified velocity interval.

"""
		NewTitle.insert(END, maintext, 'normal')
		NewTitle.insert(END, '\n')
		
		NewTitle.insert(END, 'Requirements', 'subsection')
		
		maintext = """
GLADoS produces searches every spectrum in the supplied data
cube(s) looking for pixels above the specified thresholds. This
can result in large catalogues as bright sources can be detected
in multiple pixels. To prevent the user having to check these
repeat detections, GLADoS uses the Starlink 'STILTS' package to
match adjacent detections, preserving only the pixel with the
highest S/N in each group. STILTS can be found """	
		NewTitle.insert(END, maintext, 'normal')
		
		NewTitle.insert(END, "here.", hyperlink.add(starlinklink))

		NewTitle.insert(END, '\n')
		
		maintext = """
GLADoS will attempt to automatically determine if you have STILTS
installed. If it cannot find it, you can give provide the path in
the 'configuration' section.
"""	
		NewTitle.insert(END, maintext, 'normal')
		NewTitle.insert(END, '\n')
		NewTitle.insert(END, '\n')
		
		NewTitle.insert(END, 'Output from GLADoS', 'subsection')
		maintext = """
In 'Search' mode, GLADoS will produce candidate detection tables
of the source positions in pixel coordinates and WCS.	Other modes
also produce a panel where you can review the sources and assign
them a confidence status (Accepted/Rejected/Uncertain). This will
produce new tables for each category. For each source you'll see
the spectrum, SDSS image (if available), a moment 0, PV map and a
detection map showing all the neabry pixels that met the detection
criteria.
As well as buttons to classify the sources, you'll also be given
the options to open the SDSS navigator at the current coordinates,
and also	query NED (with the search radius determined by the beam
size). There are actually two NED buttons, one which returns all
objects in NED within the search radius and one which shows only
galaxies with known redshift information.

Inspecting the detection map can be important as nearby sources may
be merged. However, at present it's far better to use Topcat to do
this. The file 'FullSearchResults.txt' contains every pixel which
satisfied the detection criteria, while 'Catalogue.txt' shows the
merged output. By manually overplotting these (Topcat can label the
merged candidates) it's possible to find galaxies that GLADoS did
detect but which got lost in the merging process. 

The default option is for GLADoS to search the cube and produce
catalogues. In 'complete' mode, once the search is complete it will
open the review panel where you can classify each source.
'Review' mode allows you to classify the detections. If this is the
first time, make sure the category is set to the default 'All'. If
you've already done at least some sources, you can set this to one
of the other options (e.g. say you want to review only the sources
you were uncertain about).
'Resume' mode is extremely useful if you were interrupted during 
the review stage. This is will let you continue reviewing from 
where you left off.

"""
		NewTitle.insert(END, maintext, 'normal')
		NewTitle.insert(END, '\n')
		
		NewTitle.insert(END, 'Advanced options', 'subsection')
		maintext = """		
The 'baseline fitting' parameters simply specify how to fit the
baseline and are unlikely to need adjusting. The 'source matching'
parameters control how the pixel-by-pixel candidates are matched to
produce the final catalogue. The 'spatial pixels' and 'velocity
channels' parameters determine how far away a pixel can be to be
considered part of the current detection group (pixels do not not
have to be contiguous).	
		
"""
		NewTitle.insert(END, maintext, 'normal')
		NewTitle.insert(END, '\n')
		
		NewTitle.insert(END, 'Saving settings', 'subsection')
		maintext = """			
GLADoS can save the settings used to a human-readable file of your
choice. Additionally, every time a search is performed it will
automatically save to a file 'ActualSettingsUsed.txt' so you will
always have a copy of the exact settings used for the search. This
file will be over-written each time a search is run.
		
"""		
		NewTitle.insert(END, maintext, 'normal')
		NewTitle.insert(END, '\n')
		
		NewTitle.insert(END, 'What if things go wrong ?', 'subsection')
		maintext = """	
In 'complete' mode, GLADoS gives warnings about the GUI being
destroyed. These are non-critical and can be ignored.

The review interface somewhat relies on the user using a standard
screen resolution of 1920x1080 pixels at something close to 80 DPI.
As long as your monitor is close to this you should not have any
major issues, though the buttons and text may not be perfectly
aligned.		
"""		
		NewTitle.insert(END, maintext, 'normal')
		NewTitle.config(state=DISABLED)
		

		NewTitle.place(x=10,y=10,width=480, heigh=480)
		
		scrollbar.config(command=NewTitle.yview)
		
		
	def go(self):
		print 'Checking values are okay...'
		goodvalues = validate()
		if goodvalues == 1:
			print 'Running operation...'
			runmaincode()


# This bit has to be here, above the calls to the main window.	
# Here we need to set all the parameters equal to what GLADOS wants, then run the main code.
def runmaincode():
	# Save settings to a file of fixed name so GLADOS can reliably call it. Also it's usefult to have
	# a definite record of what settings were actually used for the search, in case the user loaded a
	# file and made alterations and forgot to save it.
	savesettingsfile('ActualSettingsUsed.txt')
	
	# Here we need error trapping for the case of no existing files, depending on the operation mode
	# Assume needed files and directories exist by default. Use empty strings for their names, so we
	# can add to them as required.
	RequiredDirectory = True	# Directory status
	RequiredFile = True			# File status
	MissingDir = ''				# Directory name
	MissingFile = ''				# File name
	readytorun = True				# Check if it's okay to proceed. Assume true by default.
	
	
	# In all review operations, we have to have the main directory. If that doesn't exist then we're 
	# screwed.
	if ModeVar.get() == 'Resume' or ModeVar.get() == 'Review':
		if os.path.isdir(dname+'/All') == False:
			RequiredDirectory = False
			MissingDir = 'All'
		if os.path.isfile(dname+'/Catalogue.txt') == False:
			MissingFile = 'Catalogue.txt'
	
	
	# If reviewing a specific category, check also for that category
	if ModeVar.get() == 'Review' and CatrVar.get() <> 'All':
		stype = CatrVar.get()
		if os.path.isdir(dname+'/'+stype) == False:
			RequiredDirectory = False
			MissingDir = '\n'+stype
		if os.path.isfile(dname+'/'+stype+'Sources.txt') == False:
			RequiredFile = False
			MissingFile = '\n'+stype+'Sources.txt'
	
			
	# But if we're resuming all sources, we also need to have all the catalogues and directories
	if ModeVar.get() == 'Resume':
		for sourcetype in ['Accepted','Uncertain','Rejected']:
			if os.path.isdir(dname+'/'+sourcetype) == False:
				RequiredDirectory = False
				MissingDir = MissingDir+'\n'+sourcetype
			if os.path.isfile(dname+'/'+sourcetype+'Sources.txt') == False:
				RequiredFile = False
				MissingFile = MissingFile+'\n'+sourcetype+'Sources.txt'
	
	
	# Now that we know the names of the missing files and directories, if any, we can raise an error
	# window		
	if RequiredDirectory == False or RequiredFile == False:
		readytorun = False
		errortext = 'Uh-oh...'
		if RequiredDirectory == False:
			errortext = errortext+'\nMissing directories :'+MissingDir
		if RequiredFile == False:
			errortext = errortext+'\n\nMissing files :'+MissingFile
		errortext = errortext+'\n'+'\nRemember that before using RESUME or REVIEW mode, you must have first run a search !'
					
		tkMessageBox.showerror("OH CRAP !", errortext)
	
	
	# Hide the main window while we do the search (it will be disabled anyway)
	MainWindow.withdraw()
	
	
	# Only proceed if no errors were raised
	if readytorun == True:
		os.system('python GLADOSSearch.py')
	
	# Bring the main window back
	MainWindow.update()
	MainWindow.deiconify()
	


# Add the buttons to the main window
ClickButtons(MainWindow)

# Calls the main GUI display function 
MainWindow.mainloop()
